This Mathematica program numerically computes discrete CMC surfaces of
revolution in R^3, S^3 and H^3.  Here are some points on how to use it:

      1) First one chooses the initial data for the surface, including
         the choice of spaceform and mean curvature, as given below.

      2) There are two different kinds of examples here.  One kind is
         where the data has been found by hand and is all written out
         in the initial data (Examples 1 to 12) -- for this kind one
         should not run "Algorithm, STEP 3".  The other kind is where
         only two adjacent points on the profile curve are given 
         (Example 13 and onward), and then the "profile curve finder" 
         (Algorithm, STEP 3) is employed.  

      3) Depending on the case, different "confirmations" and "plotters"
         are used.  

      4) It is explained in the comments before each initial data set which 
         algorithm steps and which plotters should be used.

      5) Because of problems with the way data is stored in variables,
         it is best to completely shut down and restart this program every
         successive time one wants to run it.
         In particular, "Algorithm, STEP 3" should never be run twice in a
         single session.

      6) When running "Algorithm, STEP 3", sometimes the error "Infinite
         expression 1/0 encountered" will appear.  But this comes only from
         unallowable data being inserted into the conditions of the "While"
         command.  Because these unallowable data are ignored by the "While"
         command, and we do in fact wish to ignore them, the program is still
         running correctly in spite of this error message.  

      7) Hkappa is the mean curvature of the surface with respect the
         chosen space form with constant sectional curvature kappa.
         On the other hand, H[m,n] is twice the value of the mean curvature
         with respect to Euclidean 3-space, which is because the metric we have
         for the space forms is 4 (dx^2+dy^2+dz^2)/(1+kappa (x^2+y^2+z^2))^2.
         (So when kappa=0, the metric is 4 (dx^2+dy^2+dz^2), not the more
         typical dx^2+dy^2+dz^2.)  

(* ================================================================ *)
(* ================================================================ *)

Example 1)
   Here we show that discrete cylinders in R^3 with nonconstant vertical
   spacing are CMC.

   Run Algorithm, STEP 1 and STEP 2, but not STEP 3,
      then run Confirmation STEP 1, 
      then run Plotter 1.  

kappatemp = 0; 
Hkappatemp = alpha/18; 
r[1] = r[2] = r[3] = r[4] = r[5] = r[6] = r[7] = r[8] = r[9] = r[10] = 3; 
alphafinal = 3/2; 
h[1] = -3;
h[2] = -1;
h[3] = 0; 
h[4] = 3;
h[5] = 4;
h[6] = 8; 
h[7] = 9;
h[8] = 10;
h[9] = 15;
h[10] = 16; 
values = {{r[1], h[1]}, {r[2], h[2]}, {r[3], h[3]}, {r[4], h[4]}, 
    {r[5], h[5]}, {r[6], h[6]}, {r[7], h[7]}, {r[8], h[8]}, {r[9], h[9]}, 
    {r[10], h[10]}}; 
maxm = 9; 
ListPlot[values, PlotStyle -> PointSize[0.02]]

(* ================================================================ *)

Example 2)
   Unduloid in R^3 with constant vertical spacing.

   Run Algorithm, STEP 1 and STEP 2, but not STEP 3,
      then run Confirmation STEP 1,
      then run Plotter 1.  

kappatemp = 0; 
Hkappatemp = N[(13 - 5*Sqrt[13])/(-117 + 30*Sqrt[13])]; 
Hkappatemp = 0.7171292729553327*alpha; 
r[1] = r[2] = r[4] = r[5] = 1.; 
r[3] = r[6] = N[-3 + Sqrt[13]]; 
alphafinal = N[(3*r[3])/(r[3]^2 + 2*r[3] + 3)]; 
h[1] = -6.; 
h[2] = -3.; 
h[3] = 0.; 
h[4] = 3.; 
h[5] = 6.; 
h[6] = 9.; 
values = {{r[1], h[1]}, {r[2], h[2]}, {r[3], h[3]}, {r[4], h[4]}, 
    {r[5], h[5]}, {r[6], h[6]}}; 
maxm = 5; 
tmpA = ListPlot[values, PlotStyle -> PointSize[0.02]]; 
tmpB = ListPlot[values, PlotStyle -> PointSize[0.02], PlotJoined -> True]; 
Show[tmpA, tmpB, AspectRatio -> True, PlotRange -> {{-10, 10}, {-10, 10}}]; 

(* ================================================================ *)

Example 3)
   Delaunay surface (unduloid) in R^3 with nonconstant vertical spacing.

   Run Algorithm, STEP 1 and STEP 2, but not STEP 3,
      then run Confirmation STEP 1,
      then run Plotter 1.  

kappatemp = 0; 
Hkappatemp = 0.36933385514009265*alpha; 
alphafinal = 0.5471316123835278; 
a = 1.5597231895915273; 
r[1] = r[2] = r[5] = r[6] = 1.; 
r[3] = r[4] = r[7] = a; 
dh = (1.*Sqrt[-2*a^3 - 9*a^2 - 8*a - 1])/Sqrt[-4*a - 3]; 
h[1] = -3*dh; 
h[2] = -dh; 
h[3] = 0; 
h[4] = dh; 
h[5] = 2*dh; 
h[6] = 4*dh; 
h[7] = 5*dh; 
maxm = 6; 
values = {{r[1], h[1]}, {r[2], h[2]}, {r[3], h[3]}, {r[4], h[4]}, 
    {r[5], h[5]}, {r[6], h[6]}, {r[7], h[7]}}; 
ListPlot[values, PlotStyle -> PointSize[0.02]]

(* ================================================================ *)

Example 4)
   A simple Delaunay nodoid in R^3.  

   Run Algorithm, STEP 1 and STEP 2, but not STEP 3, 
      then run Confirmation STEP 1,
      then run Plotter 1.  

kappatemp = 0; 
Hkappatemp = -85*alpha; 
a = 1./20; 
dh = (1/3)*1.*Sqrt[-5*a^2 - 4*a + 1]; 
alpha = (2*a^2)/Sqrt[4*a^4 + 12*a^3 + 5*a^2 - 6*a + 1]; 
Clear[alpha]; 
alphafinal = 0.005917159763313611; 
r[1] = 1.; 
r[2] = a; 
r[3] = a; 
r[4] = 1.; 
r[5] = a; 
r[6] = a; 
h[1] = -2*dh; 
h[2] = dh; 
h[3] = -dh; 
h[4] = 2*dh; 
h[5] = 5*dh; 
h[6] = 3*dh; 
values = {{r[1], h[1]}, {r[2], h[2]}, {r[3], h[3]}, {r[4], h[4]}, 
    {r[5], h[5]}, {r[6], h[6]}}; 
maxm = 5; 
tmpA = ListPlot[values, PlotStyle -> PointSize[0.02]]; 
tmpB = ListPlot[values, PlotStyle -> PointSize[0.02], PlotJoined -> True]; 
Show[tmpA, tmpB, AspectRatio -> True, PlotRange -> {{-2, 2}, {-2, 2}}]; 

(* ================================================================ *)

Example 5)
   A nonconformal unduloid in R^3 with constant vertical spacing.

   Run Algorithm, STEP 1 and STEP 2, but not STEP 3,
      then run Confirmation STEP 1,
      then run Plotter 1.  

kappatemp = 0; 
Hkappatemp = 0.7600353344758148*alpha; 
alphafinal = 0.39471849056452807; 
h[1] = -2;
h[2] = -1;
h[3] = 0;
h[4] = 1;
h[5] = 2;
h[6] = 3;
h[7] = 4;
h[8] = 5;
h[9] = 6;
h[10] = 7; 
r[1] = 1; 
r[2] = 1; 
r[3] = 0.8319292834021221; 
r[4] = 0.6602651208306597; 
r[5] = 0.6864012614863299; 
r[6] = 0.8785575700510736; 
r[7] = 1.0170778610483509; 
r[8] = 0.9737356499446731; 
r[9] = 0.7847547417969657; 
r[10] = 0.6453171909236644; 
maxm = 9; 
values = {{r[1], h[1]}, {r[1], h[1]}, {r[2], h[2]}, {r[3], h[3]}, 
    {r[4], h[4]}, {r[5], h[5]}, {r[6], h[6]}, {r[7], h[7]}, {r[8], h[8]}, 
    {r[9], h[9]}, {r[10], h[10]}}; 
tmpA = ListPlot[values, PlotStyle -> PointSize[0.02]]; 
tmpB = ListPlot[values, PlotStyle -> PointSize[0.02], PlotJoined -> True]; 
Show[tmpA, tmpB]; 

(* ================================================================ *)

Example 6)
   A surface in R^3 with constant hyperbolic speed ("quasi-conformal")
   parametrization, i.e., the cross ratios of the quadrilaterals are constant.
   This is a kind of conformality condition.
   (True conformality is that the cross ratio is always exactly -1.)
   
   Run Algorithm, STEP 1 and STEP 2, but not STEP 3,
      then run Confirmation STEP 1,
      then run Plotter 1.  

kappatemp = 0; 
Hkappatemp = 0.7556900947358763*alpha; 
alphafinal = 0.39698813321729953; 
h[1] = 1;
h[2] = 2; 
r[1] = 1;
r[2] = 0.98; 
r[3] = 0.8278314829645018;
h[3] = 2.897222022679383; 
r[4] = 0.6896713584501286;
h[4] = 3.6480403384859295; 
r[5] = 0.6551620524548313;
h[5] = 4.326318596182285; 
r[6] = 0.7400441029543396;
h[6] = 5.024700026433265; 
r[7] = 0.900805007427301;
h[7] = 5.833815757410824; 
r[8] = 1.0114478471722468;
h[8] = 6.791856237985496; 
r[9] = 0.9539048443884937;
h[9] = 7.78261247256889; 
r[10] = 0.7926191411924897;
h[10] = 8.646214832490255; 
maxm = 9; 
values = {{r[1], h[1]}, {r[1], h[1]}, {r[2], h[2]}, {r[3], h[3]}, 
    {r[4], h[4]}, {r[5], h[5]}, {r[6], h[6]}, {r[7], h[7]}, {r[8], h[8]}, 
    {r[9], h[9]}, {r[10], h[10]}}; 
tmpA = ListPlot[values, PlotStyle -> PointSize[0.02]]; 
tmpB = ListPlot[values, PlotStyle -> PointSize[0.02], PlotJoined -> True]; 
Show[tmpA, tmpB]; 

(* ================================================================ *)

Example 7)
   A surface in S^3 with constant hyperbolic speed ("quasi-conformal")
   parametrization.

   Run Algorithm, STEP 1 and STEP 2, but not STEP 3,
      then run Confirmation STEP 1,
      then run Plotter 1, but not Plotter 2.

kappatemp = 1; 
Hkappatemp = 1.8511673740392585*alpha; 
alphafinal = 0.16205990025926073; 
h[1] = -0.5;
h[2] = 0; 
r[1] = 0.3;
r[2] = 0.4; 
r[3] = 0.676607279854556448186258283922905489532383995638; 
h[3] = 0.714059793438485221012315999753268362514013089422; 
r[4] = 1.673564231208823394018947420526; 
h[4] = 1.922154413815239832714628596881; 
r[5] = 3.799688952583472569830350594887; 
h[5] = -1.12045597270257928767116100748510893734241944304; 
r[6] = 0.974663787195755603465398010080452404278192424048; 
h[6] = -0.912350706655932850315225642529457494162745069113; 
maxm = 5; 
values = {{r[1], h[1]}, {r[1], h[1]}, {r[2], h[2]}, {r[3], h[3]}, 
    {r[4], h[4]}, {r[5], h[5]}, {r[6], h[6]}}; 
tmpA = ListPlot[values, PlotStyle -> PointSize[0.02]]; 
tmpB = ListPlot[values, PlotStyle -> PointSize[0.02], PlotJoined -> True]; 
Show[tmpA, tmpB]; 

(* ================================================================ *)

Example 8)
   A nonconformal minimal surface in S^3.

   Run Algorithm, STEP 1 and STEP 2, but not STEP 3,
      then run Confirmation STEP 1,
      then run Plotter 1, but not Plotter 2.

kappatemp = 1; 
Hkappatemp = 0; 
alphafinal = 0.11560343109274111; 
h[1] = -0.4;
h[2] = -0.35;
h[3] = -0.3;
h[4] = -0.25;
h[5] = -0.2; 
h[6] = -0.15;
h[7] = -0.1;
h[8] = -0.05;
h[9] = 0;
h[10] = 0.05; 
r[1] = 0.35;
r[2] = 0.4;
r[3] = 0.450837769534866; 
r[4] = 0.5011552699167381;
r[5] = 0.5500984707962404; 
r[6] = 0.5971849989144771; 
r[7] = 0.642177979256725;
r[8] = 0.684992149840414; 
r[9] = 0.7256314751893436;
r[10] = 0.7641500431455006; 
maxm = 9; 
values = {{r[1], h[1]}, {r[1], h[1]}, {r[2], h[2]}, {r[3], h[3]}, 
    {r[4], h[4]}, {r[5], h[5]}, {r[6], h[6]}, {r[7], h[7]}, {r[8], h[8]}, 
    {r[9], h[9]}, {r[10], h[10]}}; 
tmpA = ListPlot[values, PlotStyle -> PointSize[0.02]]; 
tmpB = ListPlot[values, PlotStyle -> PointSize[0.02], PlotJoined -> True]; 
Show[tmpA, tmpB]; 

(* ================================================================ *)

Example 9)
   Here we see that discrete cylinders in S^3 with nonconstant vertical
   spacing are CMC.

   Run Algorithm, STEP 1 and STEP 2, but not STEP 3,
      then run Confirmation STEP 1,
      then run Plotter 1, but not Plotter 2.

kappatemp = 1; 
Hkappatemp = 4.*alpha; 
alphafinal = 0.14433756729740646; 
dh = 0.05; 
rr = 2.; 
alpha = 1/Sqrt[16*(rr^2 - 1)]; 
Clear[alpha]; 
h[1] = -3*dh; 
h[2] = -dh; 
h[3] = 0.; 
h[4] = 3*dh; 
h[5] = 4*dh; 
h[6] = 8*dh; 
r[1] = rr - Sqrt[rr^2 - h[1]^2 - 1]; 
r[2] = rr - Sqrt[rr^2 - h[2]^2 - 1]; 
r[3] = rr - Sqrt[rr^2 - h[3]^2 - 1]; 
r[4] = rr - Sqrt[rr^2 - h[4]^2 - 1]; 
r[5] = rr - Sqrt[rr^2 - h[5]^2 - 1]; 
r[6] = rr - Sqrt[rr^2 - h[6]^2 - 1]; 
values = {{r[1], h[1]}, {r[2], h[2]}, {r[3], h[3]}, {r[4], h[4]}, 
    {r[5], h[5]}, {r[6], h[6]}}; 
maxm = 5; 
ListPlot[values, PlotStyle -> PointSize[0.02]]

(* ================================================================ *)

Example 10)
   Here we see that discrete cylinders in H^3 with nonconstant vertical
   spacing are CMC.

   Run Algorithm, STEP 1 and STEP 2, but not STEP 3,
      then run Confirmation STEP 1,
      then run Plotter 1.

kappatemp = -1; 
Hkappatemp = 12.*alpha; 
dh = 0.1; 
rr = 2; 
alphafinal = N[1/Sqrt[16*(rr^2 + 1)]]; 
h[1] = -3*dh;
h[2] = -dh;
h[3] = 0.;
h[4] = 3*dh;
h[5] = 4*dh;
h[6] = 8*dh; 
r[1] = Sqrt[rr^2 - h[1]^2 + 1] - rr;
r[2] = Sqrt[rr^2 - h[2]^2 + 1] - rr; 
r[3] = Sqrt[rr^2 - h[3]^2 + 1] - rr; 
r[4] = Sqrt[rr^2 - h[4]^2 + 1] - rr;
r[5] = Sqrt[rr^2 - h[5]^2 + 1] - rr; 
r[6] = Sqrt[rr^2 - h[6]^2 + 1] - rr; 
maxm = 5; 
values = {{r[1], h[1]}, {r[2], h[2]}, {r[3], h[3]}, {r[4], h[4]}, 
    {r[5], h[5]}, {r[6], h[6]}}; 
ListPlot[values, PlotStyle -> PointSize[0.02]]

(* ================================================================ *)

Example 11)
   A nonconformal Delaunay surface in H^3, with 0<|H|<1.

   Run Algorithm, STEP 1 and STEP 2, but not STEP 3,
      then run Confirmation STEP 1,
      then run Plotter 1.

kappatemp = -1; 
Hkappatemp = 1.3031075978515285*alpha; 
alphafinal = 0.23021890171970316; 
h[1] = -0.4;
h[2] = -0.35;
h[3] = -0.3;
h[4] = -0.25;
h[5] = -0.2; 
h[6] = -0.15;
h[7] = -0.1;
h[8] = -0.05;
h[9] = 0;
h[10] = 0.05; 
r[1] = 0.2;
r[2] = 0.2; 
r[3] = 0.21164740802233467;
r[4] = 0.2363928805808025; 
r[5] = 0.27644652948778875;
r[6] = 0.3349874804564269; 
r[7] = 0.41568908856969844;
r[8] = 0.5202208797952523; 
r[9] = 0.6432723840612092;
r[10] = 0.7716179609185361; 
maxm = 9; 
values = {{r[1], h[1]}, {r[1], h[1]}, {r[2], h[2]}, {r[3], h[3]}, 
    {r[4], h[4]}, {r[5], h[5]}, {r[6], h[6]}, {r[7], h[7]}, {r[8], h[8]}, 
    {r[9], h[9]}, {r[10], h[10]}}; 
ListPlot[values, PlotStyle -> PointSize[0.02]]; 

(* ================================================================ *)

Example 12)
   Another nonconformal Delaunay surface in H^3, with |H|>1.

   Run Algorithm, STEP 1 and STEP 2, but not STEP 3,
      then run Confirmation STEP 1, 
      then run Plotter 1.

kappatemp = -1; 
Hkappatemp = -4.550303810904895*alpha; 
alphafinal = 0.4395310913541575; 
h[1] = -0.4;
h[2] = -0.35;
h[3] = -0.3;
h[4] = -0.25;
h[5] = -0.2; 
h[6] = -0.25;
h[7] = -0.3;
h[8] = -0.25;
h[9] = -0.2;
h[10] = -0.15; 
r[1] = 0.3;
r[2] = 0.2; 
r[3] = 0.41632832255236624;
r[4] = 0.4103884056542382; 
r[5] = 0.38551144570905044;
r[6] = 0.19918519913636973; 
r[7] = 0.41879722780040757;
r[8] = 0.4275403844455409; 
r[9] = 0.42089106382521874;
r[10] = 0.2253042651521808; 
maxm = 9; 
values = {{r[1], h[1]}, {r[1], h[1]}, {r[2], h[2]}, {r[3], h[3]}, 
    {r[4], h[4]}, {r[5], h[5]}, {r[6], h[6]}, {r[7], h[7]}, {r[8], h[8]}, 
    {r[9], h[9]}, {r[10], h[10]}}; 
tmpA = ListPlot[values, PlotStyle -> PointSize[0.02]]; 
tmpB = ListPlot[values, PlotStyle -> PointSize[0.02], PlotJoined -> True]; 
Show[tmpA, tmpB]; 

(* ================================================================ *)

Example 13)
   This is a discrete cylinder in R^3 with constant vertical spacing.

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1.

kappatemp = 0; 
Hkappatemp = 0.25; 
rtemp[1] = 1.; 
rtemp[2] = 1.; 
htemp[1] = 0.; 
htemp[2] = 1.; 
maxmtemp = 15; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 14)
   Now for a noncylindrical Delaunay unduloid in R^3 with nonconstant
   vertical spacing.

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1.  

kappatemp = 0; 
Hkappatemp = 0.2845858468364701; 
htemp[1] = 3.; 
htemp[2] = 6.; 
rtemp[1] = 1; 
rtemp[2] = 1; 
maxmtemp = 30; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 15)
   A nodoid in R^3.

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1.  

kappatemp = 0; 
Hkappatemp = -0.3; 
htemp[1] = -2; 
htemp[2] = -1; 
rtemp[1] = 1; 
rtemp[2] = 1; 
maxmtemp = 20; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 16)
   An unduloid in R^3.

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1.

kappatemp = 0; 
Hkappatemp = 1.; 
htemp[1] = -0.025; 
htemp[2] = 0.025; 
rtemp[1] = 0.3; 
rtemp[2] = 0.3; 
maxmtemp = 100; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 17)
   A nodoid in R^3.

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1.  

kappatemp = 0; 
Hkappatemp = -1.; 
htemp[1] = -0.025; 
htemp[2] = 0.025; 
rtemp[1] = 0.3; 
rtemp[2] = 0.3; 
maxmtemp = 100; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 18)
   A constant hyperbolic speed ("quasi-conformal") parametrization,
   a catenoid in R^3 (it turns out that all dh_{m+1,m} is constant,
   i.e. independent of m).  
   Note that here we will be adding the condition that the cross ratios
   of the quadrilaterals are a constant.  This is a kind of conformality
   condition.  
   (True conformality is that the cross ratio is always exactly -1.)

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1.  

kappatemp = 0; 
Hkappatemp = 0.; 
htemp[1] = -0.025; 
htemp[2] = 0.025; 
rtemp[1] = 1.; 
rtemp[2] = 1.; 
maxmtemp = 102; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 19)
   A constant hyperbolic speed ("quasi-conformal") parametrization of a
   Delaunay surface in S^3 that closes on itself to become an embedded torus.
   Note that here we will be adding the condition that the cross ratios of the 
   quadrilaterals are a constant.  This is a kind of conformality condition.  
   (True conformality is that the cross ratio is always exactly -1.)

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1 and Plotter 2.  

kappatemp = 1; 
Hkappatemp = 1.3485; 
htemp[1] = -0.049947; 
htemp[2] = 0.049947; 
rtemp[1] = 0.3; 
rtemp[2] = 0.3; 
maxmtemp = 102; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 20)
   A minimal example in H^3.

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1 and Plotter 3 and Plotter 4.  

kappatemp = -1; 
Hkappatemp = 0.; 
rtemp[1] = 0.9913991417; 
rtemp[2] = 1; 
htemp[1] = -0.005; 
htemp[2] = 0.; 
maxmtemp = 40; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 21)
   A minimal example in H^3.

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1 and Plotter 3 and Plotter 4.  

kappatemp = -1; 
Hkappatemp = 0.; 
rtemp[1] = 1.281765730881669683938008123909; 
rtemp[2] = 1.469728846958890357130788321033; 
htemp[1] = 0.782997389364556374502637426831; 
htemp[2] = 1.061963485893203535160632851315; 
maxmtemp = 90; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 22)
   A minimal example in H^3.

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1 and Plotter 3 and Plotter 4.  

kappatemp = -1; 
Hkappatemp = 0.; 
rtemp[1] = 1.056565730881669683938008123909; 
rtemp[2] = 1.069728846958890357130788321033; 
htemp[1] = 0.782997389364556374502637426831; 
htemp[2] = 1.061963485893203535160632851315; 
maxmtemp = 26; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 23)
   A minimal example in H^3.

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1 and Plotter 3 and Plotter 4.  

kappatemp = -1; 
Hkappatemp = 0.; 
rtemp[1] = 1.090565730881669683938008123909; 
rtemp[2] = 1.069728846958890357130788321033; 
htemp[1] = 0.782997389364556374502637426831; 
htemp[2] = 1.061963485893203535160632851315; 
maxmtemp = 35; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 24)
   Next we give an example in H^3 with the constant value of the mean
   curvature lying strictly between 0 and 1.

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1 and Plotter 3 and Plotter 4.  

kappatemp = -1; 
Hkappatemp = 0.3; 
htemp[1] = -0.4; 
htemp[2] = -0.35; 
rtemp[1] = 0.2; 
rtemp[2] = 0.2; 
maxmtemp = 21; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 25)
   An H^3 example with the constant value of the mean curvature strictly
   greater than 1.

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1 and Plotter 3 and Plotter 4.  

kappatemp = -1; 
Hkappatemp = 2; 
htemp[1] = -0.4; 
htemp[2] = -0.35; 
rtemp[1] = 0.3; 
rtemp[2] = 0.2; 
maxmtemp = 10; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 26)
   A constant hyperbolic speed ("quasi-conformal") parametrization,
   an unduloid in H^3, with the constant value of the mean curvature
   strictly greater than 1:  
   Note that here we will be adding the condition that the cross ratios
   of the quadrilaterals are a constant.  This is a kind of conformality
   condition.
   (True conformality is that the cross ratio is always exactly -1.)

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1 and Plotter 3 and Plotter 4.  

kappatemp = -1; 
Hkappatemp = 1.3; 
htemp[1] = -0.025; 
htemp[2] = 0.025; 
rtemp[1] = 0.3; 
rtemp[2] = 0.3; 
maxmtemp = 50; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 27)
   A constant hyperbolic speed ("quasi-conformal") parametrization,
   an embedded catenoid cousin in H^3 (with mean curvature exactly 1).
   Note that here we will be adding the condition that the cross ratios
   of the quadrilaterals are a constant.  This is a kind of conformality
   condition.  
   (True conformality is that the cross ratio is always exactly -1.)

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1 and Plotter 3 and Plotter 4.  

kappatemp = -1; 
Hkappatemp = 1.; 
htemp[1] = -0.025; 
htemp[2] = 0.025; 
rtemp[1] = 0.3; 
rtemp[2] = 0.3; 
maxmtemp = 50; 
choiceofalpha = 2; 

(* ================================================================ *)

Example 28)
   A constant hyperbolic speed ("quasi-conformal") parametrization,
   an embedded minimal rotational Willmore channel surface in H^3
   (see page 141 of the book by Udo Hertrich-Jeromin).  
   Note that here we will be adding the condition that the cross ratios
   of the quadrilaterals are a constant.  This is a kind of conformality
   condition.  
   (True conformality is that the cross ratio is always exactly -1.)

   Run Algorithm, all three steps: STEP 1 and STEP 2 and STEP 3,
      then run Confirmation STEP 2,
      then run Plotter 1 and Plotter 3 and Plotter 4.  

kappatemp = -1; 
Hkappatemp = 0.; 
htemp[1] = -0.025; 
htemp[2] = 0.025; 
rtemp[1] = 0.3; 
rtemp[2] = 0.3; 
maxmtemp = 100; 
choiceofalpha = 2; 

(* ================================================================ *)
(* ================================================================ *)
(* ================================================================ *)

Algorithm, STEP 1)
   Here we define the matrix for the linear system in the upcoming algorithm:

bigmatrix[m_] := {
   {r[m]^2, 0, 0, 0, r[m], 0, 0, alpha*(kappa*(h[m]^2+r[m]^2)+1)},
   {0, r[m+1]^2, 0, 0, 0, r[m+1], 0, alpha*(kappa*(h[m+1]^2+r[m+1]^2)+1)},
   {0, 0, h[m+1]-h[m], h[m+1]-h[m], r[m+1]-r[m], r[m+1]-r[m], 0, 0},
   {0, 0, -(r[m+1]-r[m]), r[m+1]-r[m], h[m+1]-h[m], -(h[m+1]-h[m]), 0, 0},
   {-1, 1, 0, 0, 0, 0, 0,
      (alpha*(kappa*(h[m+1]^2-h[m]^2+r[m+1]^2-r[m]^2)))/(r[m]*r[m+1])},
   {(1/2)*((h[m+1]-h[m])^2+(r[m+1]-r[m])^2),
      (1/2)*((h[m+1]-h[m])^2+(r[m+1]-r[m])^2),
      -(h[m+1]-h[m]), h[m+1]-h[m], -(r[m+1]-r[m]), r[m+1]-r[m], 0,
      -((alpha*(((1/2)*kappa*(h[m+1]^2+h[m]^2+r[m+1]^2+r[m]^2)+1)*
         ((h[m+1]-h[m])^2+(r[m+1]-r[m])^2)))/(r[m]*r[m+1]))},
   {kappa*(h[m]^2+r[m]^2)+1, 0, 2*kappa*h[m], 0, 2*kappa*r[m], 0, -2, 0},
   {0, kappa*(h[m+1]^2+r[m+1]^2)+1, 0, 2*kappa*h[m+1], 0, 2*kappa*r[m+1], -2, 0}
   }; 

(* ================================================================ *)

Algorithm, STEP 2)
   Here we define some other needed terms
   (a single 0 should appear in the output):

xxx[m_] := Simplify[Solve[
   bigmatrix[m] . {
      {H[m,n]}, {H[m+1,n]}, {nneta[m,n]}, {nneta[m+1,n]},
      {nnrho[m,n]}, {nnrho[m+1,n]}, {Hkappa}, {1}
      } == {{0}, {0}, {0}, {0}, {0}, {0}, {0}, {0}},
   {H[m,n], H[m+1,n], nneta[m,n], nneta[m+1,n], nnrho[m,n], nnrho[m+1,n]}
   ]]; 

Chop[Simplify[
   -(nneta[m+1,n] /. xxx[m])^2 + (nneta[m,n] /. xxx[m])^2
   - (nnrho[m+1,n] /. xxx[m])^2 + (nnrho[m,n] /. xxx[m])^2
   ][[1]]]

nnetaplus[m_,n_]  = (nneta[m,n] /. xxx[m])[[1]]; 
nnetaminus[m_,n_] = (nneta[m,n] /. xxx[m-1])[[1]]; 
nnrhoplus[m_,n_]  = (nnrho[m,n] /. xxx[m])[[1]]; 
nnrhominus[m_,n_] = (nnrho[m,n] /. xxx[m-1])[[1]]; 

Hplus[m_,n_]  = (H[m,n] /. xxx[m])[[1]]; 
Hminus[m_,n_] = (H[m,n] /. xxx[m-1])[[1]]; 

InputForm[nnetadiff[m_,n_] = Simplify[nnetaplus[m,n] - nnetaminus[m,n]]]; 
InputForm[nnrhodiff[m_,n_] = Simplify[nnrhoplus[m,n] - nnrhominus[m,n]]]; 
InputForm[Hdiff[m_,n_] = Simplify[Hplus[m,n] - Hminus[m,n]]]; 

kappa  = kappatemp; 
Hkappa = Hkappatemp; 

(* ================================================================ *)

Algorithm, STEP 3)
   Here is the algorithm that determines the profile curve numerically
   (the term "crossratio" is not the actual cross ratio, it differs
   from the true cross ratio by a constant factor):

r[1] = rtemp[1]; 
r[2] = rtemp[2]; 
h[1] = htemp[1]; 
h[2] = htemp[2]; 
maxm = maxmtemp; 

Chop[Simplify[
   nnetaminus[2,n]^2 + nnrhominus[2,n]^2-1
   ]]

tmp = NSolve[Chop[Simplify[
   nnetaplus[1,n]^2 + nnrhoplus[1,n]^2 - 1
   ]], alpha]
alpha = (alpha /. tmp)[[choiceofalpha]];
Print["alpha is ", alpha]; 

crossratio = Chop[Simplify[
   ((h[2]-h[1])^2 + (r[2]-r[1])^2)/(r[1]*r[2])
   ]];
Print["crossratio is ", crossratio]; 

Chop[Simplify[
   nnetaminus[2,n]^2 + nnrhominus[2,n]^2 - 1
   ]]

Chop[Simplify[
   nnetaplus[1,n]^2 + nnrhoplus[1,n]^2 - 1
   ]]

Do[ { tmp = NSolve[
      {
         Chop[Simplify[ nnetadiff[m,n] ]] == 0, 
         Chop[Simplify[
            (h[m+1]-h[m])^2 + (r[m+1]-r[m])^2 - crossratio*r[m]*r[m+1]
            ]] == 0
      },
      { r[m + 1], h[m + 1] }
      ];
      j = 1; 
      rrr[m+1] = (r[m+1] /. tmp)[[j]];
      hhh[m+1] = (h[m+1] /. tmp)[[j]];
      While[
         Abs[Im[rrr[m+1]]] > 0.0001 || 
            Abs[(hhh[m+1]-h[m-1])^2 + (rrr[m+1]-r[m-1])^2] < 1.*^-7 || 
            Re[rrr[m+1]] < 0.0001, 
         {
            j = j+1;
            rrr[m+1] = (r[m+1] /. tmp)[[j]];
            hhh[m+1] = (h[m+1] /. tmp)[[j]];
         }
      ]; 
      r[m+1] = (r[m+1] /. tmp)[[j]];
      h[m+1] = (h[m+1] /. tmp)[[j]];
    }, {m, 2, maxm} ]; 

values = Table[{r[jj], h[jj]}, {jj, 1, maxm+1}]; 

MoebiusTransformedValues = Table[ {
   (2*r[jj])/((h[jj] + 1)^2 + r[jj]^2), 
   (h[jj]^2 + r[jj]^2 - 1)/((h[jj] + 1)^2 + r[jj]^2)
   }, {jj, 1, maxm+1} ]; 
MoebiusTransformedValuesS3 = Table[ {
   (h[jj]^2 + r[jj]^2 - 1)/((r[jj] + 1)^2 + h[jj]^2), 
   (2*h[jj])/((r[jj] + 1)^2 + h[jj]^2)
   }, {jj, 1, maxm+1} ]; 

tmpA = ListPlot[values, PlotStyle -> PointSize[0.005]]; 
tmpB = ListPlot[values, PlotStyle -> PointSize[0.005], PlotJoined -> True]; 

Show[tmpA, tmpB, PlotRange -> All]; 

(* ================================================================ *)
(* ================================================================ *)

Confirmation STEP 1)
   We confirm that the above numerics is correct here.
   Everything should be zero, except the last numbers, which are the
   mean curvature w.r.t. Euclidean space (printed twice) and then w.r.t.
   the chosen space form:

Do[Print[
   Simplify[Chop[nnetadiff[m,n], 0.00001]], "   ", 
   Simplify[Chop[nnrhodiff[m,n], 0.00001]], "   ", 
   Simplify[Chop[Hdiff[m,n], 0.00001]], "   ", 
   Transpose[Chop[Simplify[
      bigmatrix[m] . {
         {Hplus[m,n]}, {Hplus[m+1,n]}, {nnetaplus[m,n]}, {nnetaplus[m+1,n]},
         {nnrhoplus[m,n]}, {nnrhoplus[m+1,n]}, {Hkappa}, {1}
         } ], 0.00001]], "   ",
   Simplify[Chop[(1/2)*Hplus[m,n] /. {alpha -> alphafinal}, 0.00001]], "   ",
   Simplify[Chop[
      Hplus[m,n]/(Sqrt[nnetaplus[m,n]^2 + nnrhoplus[m,n]^2]*2)
      /. {alpha -> alphafinal}, 0.00001]], "   ", 
   Simplify[Chop[Hkappa /. {alpha -> alphafinal}, 0.00001]]
   ], {m, 2, maxm-1}];

(* ================================================================ *)

Confirmation STEP 2)
   We confirm that the above numerics is correct here.
   Everything should be zero, except the last numbers, which are the
   mean curvature w.r.t. Euclidean space (printed twice) and then w.r.t.
   the chosen space form:

Do[Print[
   Simplify[Chop[nnetadiff[m, n], 0.00001]], "   ",
   Simplify[Chop[nnrhodiff[m, n], 0.00001]], "   ",
   Simplify[Chop[Hdiff[m, n], 0.00001]], "   ",
   Transpose[Chop[Simplify[
      bigmatrix[m] . {
         {Hplus[m,n]}, {Hplus[m+1,n]}, {nnetaplus[m,n]}, {nnetaplus[m+1,n]},
         {nnrhoplus[m,n]}, {nnrhoplus[m+1,n]}, {Hkappa}, {1}
         } ], 0.00001]], "   ",
   Simplify[Chop[(1/2)*Hplus[m,n], 0.00001]], "   ",
   Simplify[Chop[
      Hplus[m,n]/(Sqrt[nnetaplus[m,n]^2+nnrhoplus[m,n]^2]*2), 0.00001]], "   ",
   Simplify[Chop[Hkappa, 0.00001]]
   ], {m, 2, maxm-1}]; 

(* ================================================================ *)
(* ================================================================ *)

Plotter 1)
   We draw the full surface here
   (Poincare ball model for H^3,
      stereographic projection model for S^3, and
      the usual vector space for R^3):

Show[Table[Graphics3D[Polygon[ {
   {values[[j]][[1]]*Cos[(k*Pi)/15],
      values[[j]][[1]]*Sin[(k*Pi)/15],
      values[[j]][[2]]},
   {values[[j]][[1]]*Cos[(1/15)*(k+1)*Pi],
      values[[j]][[1]]*Sin[(1/15)*(k+1)*Pi],
      values[[j]][[2]]}, 
   {values[[j+1]][[1]]*Cos[(1/15)*(k+1)*Pi],
      values[[j+1]][[1]]*Sin[(1/15)*(k+1)*Pi],
      values[[j+1]][[2]]}, 
   {values[[j+1]][[1]]*Cos[(k*Pi)/15],
      values[[j+1]][[1]]*Sin[(k*Pi)/15],
      values[[j+1]][[2]]}
   } ]], {j, 1, maxm-3}, {k, 0, 14}],
      PlotRange -> All, AspectRatio -> Automatic,
      Boxed -> False, ViewPoint -> {4., -5., 2.}]; 

(* ================================================================ *)

Plotter 2)
   The following is only for moving the picture of
   the profile curve by an isometry of S^3,
   so should only be used in the S^3 case
   (it makes the picture easier to "see"):

tmpAMoebiusTransformedS3 = ListPlot[
   MoebiusTransformedValuesS3,
   PlotStyle -> PointSize[0.005]]; 
tmpBMoebiusTransformedS3 = ListPlot[
   MoebiusTransformedValuesS3, 
   PlotStyle -> PointSize[0.005], PlotJoined -> True]; 

Show[
   tmpAMoebiusTransformedS3, tmpBMoebiusTransformedS3,
   PlotRange -> All, AspectRatio -> True]; 

(* ================================================================ *)

Plotter 3)
   The following is only for moving the picture of the profile curve into
   the upper and lower half space models for the two copies of H^3,
   so should only be used in the H^3 case:

tmpAMoebiusTransformed = ListPlot[
   MoebiusTransformedValues, 
   PlotStyle -> PointSize[0.005]]; 
tmpBMoebiusTransformed = ListPlot[
   MoebiusTransformedValues, 
   PlotStyle -> PointSize[0.005], PlotJoined -> True]; 

Show[
   tmpAMoebiusTransformed, tmpBMoebiusTransformed,
   PlotRange -> All, AspectRatio -> True]; 

(* ================================================================ *)

Plotter 4)
   In the case of H^3, we draw the full surface here,
   using the upper and lower half space models:

firstpart = Show[Table[Graphics3D[Polygon[ {
   {MoebiusTransformedValues[[j]][[1]]*Cos[(k*Pi)/15],
      MoebiusTransformedValues[[j]][[1]]*Sin[(k*Pi)/15], 
      MoebiusTransformedValues[[j]][[2]]}, 
   {MoebiusTransformedValues[[j]][[1]]*Cos[(1/15)*(k+1)*Pi], 
      MoebiusTransformedValues[[j]][[1]]*Sin[(1/15)*(k+1)*Pi], 
      MoebiusTransformedValues[[j]][[2]]}, 
   {MoebiusTransformedValues[[j+1]][[1]]*Cos[(1/15)*(k+1)*Pi], 
      MoebiusTransformedValues[[j+1]][[1]]*Sin[(1/15)*(k+1)*Pi], 
      MoebiusTransformedValues[[j+1]][[2]]}, 
   {MoebiusTransformedValues[[j+1]][[1]]*Cos[(k*Pi)/15], 
      MoebiusTransformedValues[[j+1]][[1]]*Sin[(k*Pi)/15], 
      MoebiusTransformedValues[[j+1]][[2]]}
   } ]], {j, 1, maxm-3}, {k, 0, 14}],
      PlotRange -> All, AspectRatio -> Automatic,
      Boxed -> False, ViewPoint -> {4., -5., 2.}]

b = 0.000001;
a = 0.05;
rad = 1.;

secondpart = Show[Graphics3D[Polygon[ {
   {rad*Cos[ (0*Pi)/28], rad*Sin[ (0*Pi)/28], b},
   {rad*Cos[     Pi/28], rad*Sin[     Pi/28], b},
   {rad*Cos[ (2*Pi)/28], rad*Sin[ (2*Pi)/28], b},
   {rad*Cos[ (3*Pi)/28], rad*Sin[ (3*Pi)/28], b},
   {rad*Cos[ (4*Pi)/28], rad*Sin[ (4*Pi)/28], b}, 
   {rad*Cos[ (5*Pi)/28], rad*Sin[ (5*Pi)/28], b},
   {rad*Cos[ (6*Pi)/28], rad*Sin[ (6*Pi)/28], b},
   {rad*Cos[ (7*Pi)/28], rad*Sin[ (7*Pi)/28], b},
   {rad*Cos[ (8*Pi)/28], rad*Sin[ (8*Pi)/28], b},
   {rad*Cos[ (9*Pi)/28], rad*Sin[ (9*Pi)/28], b},
   {rad*Cos[(10*Pi)/28], rad*Sin[(10*Pi)/28], b},
   {rad*Cos[(11*Pi)/28], rad*Sin[(11*Pi)/28], b},
   {rad*Cos[(12*Pi)/28], rad*Sin[(12*Pi)/28], b},
   {rad*Cos[(13*Pi)/28], rad*Sin[(13*Pi)/28], b},
   {rad*Cos[(14*Pi)/28], rad*Sin[(14*Pi)/28], b},
   {rad*Cos[(15*Pi)/28], rad*Sin[(15*Pi)/28], b},
   {rad*Cos[(16*Pi)/28], rad*Sin[(16*Pi)/28], b},
   {rad*Cos[(17*Pi)/28], rad*Sin[(17*Pi)/28], b},
   {rad*Cos[(18*Pi)/28], rad*Sin[(18*Pi)/28], b},
   {rad*Cos[(19*Pi)/28], rad*Sin[(19*Pi)/28], b},
   {rad*Cos[(20*Pi)/28], rad*Sin[(20*Pi)/28], b},
   {rad*Cos[(21*Pi)/28], rad*Sin[(21*Pi)/28], b},
   {rad*Cos[(22*Pi)/28], rad*Sin[(22*Pi)/28], b},
   {rad*Cos[(23*Pi)/28], rad*Sin[(23*Pi)/28], b},
   {rad*Cos[(24*Pi)/28], rad*Sin[(24*Pi)/28], b},
   {rad*Cos[(25*Pi)/28], rad*Sin[(25*Pi)/28], b},
   {rad*Cos[(26*Pi)/28], rad*Sin[(26*Pi)/28], b},
   {rad*Cos[(27*Pi)/28], rad*Sin[(27*Pi)/28], b},
   {rad*Cos[(28*Pi)/28], rad*Sin[(28*Pi)/28], b}
   } ]],
      PlotRange -> All, AspectRatio -> Automatic,
      Boxed -> False, ViewPoint -> {4., -5., 2.}]; 

figure = Show[
   firstpart, secondpart,
   ViewPoint -> {6., -7., 1.5}]

(* ================================================================ *)
(* ================================================================ *)


